@@ -1,4 +1,5 @@
 use std::fs::File;
+use std::time::SystemTime;
 
 use arrow2::error::Result;
 use arrow2::io::parquet::read;
@@ -12,9 +13,11 @@ fn main() -> Result<()> {
     let reader = File::open(file_path)?;
     let reader = read::RecordReader::try_new(reader, None, None, None, None)?;
 
-    for maybe_batch in reader {
-        let batch = maybe_batch?;
-        println!("{:?}", batch);
+    let start = SystemTime::now();
+    for maybe_chunk in reader {
+        let columns = maybe_chunk?;
+        assert!(!columns.is_empty());
     }
+    println!("took: {} ms", start.elapsed().unwrap().as_millis());
     Ok(())
 }
