@@ -3,24 +3,22 @@ use std::sync::Arc;
 
 use arrow2::{
     array::{Array, Int32Array},
+    chunk::Chunk,
     datatypes::{Field, Schema},
     error::Result,
     io::parquet::write::{
         write_file, Compression, Encoding, RowGroupIterator, Version, WriteOptions,
     },
-    record_batch::RecordBatch,
 };
 
-fn write_batch(path: &str, batch: RecordBatch) -> Result<()> {
-    let schema = batch.schema().clone();
-
+fn write_batch(path: &str, schema: Schema, columns: Chunk<Arc<dyn Array>>) -> Result<()> {
     let options = WriteOptions {
         write_statistics: true,
         compression: Compression::Uncompressed,
         version: Version::V2,
     };
 
-    let iter = vec![Ok(batch)];
+    let iter = vec![Ok(columns)];
 
     let row_groups =
         RowGroupIterator::try_new(iter.into_iter(), &schema, options, vec![Encoding::Plain])?;
@@ -52,8 +50,8 @@ fn main() -> Result<()> {
         Some(6),
     ]);
     let field = Field::new("c1", array.data_type().clone(), true);
-    let schema = Schema::new(vec![field]);
-    let batch = RecordBatch::try_new(Arc::new(schema), vec![Arc::new(array)])?;
+    let schema = Schema::from(vec![field]);
+    let columns = Chunk::new(vec![Arc::new(array) as Arc<dyn Array>]);
 
-    write_batch("test.parquet", batch)
+    write_batch("test.parquet", schema, columns)
 }
