@@ -1,8 +1,9 @@
 use crate::{bitmap::Bitmap, buffer::Buffer, datatypes::DataType};
 
 use super::{
-    display_fmt, display_helper, specification::check_offsets,
-    specification::check_offsets_minimal, specification::Offset, Array, GenericBinaryArray,
+    display_fmt, display_helper,
+    specification::{check_offsets, check_offsets_minimal},
+    Array, GenericBinaryArray, Offset,
 };
 
 mod ffi;
@@ -29,7 +30,12 @@ pub struct BinaryArray<O: Offset> {
 impl<O: Offset> BinaryArray<O> {
     /// Creates an empty [`BinaryArray`], i.e. whose `.len` is zero.
     pub fn new_empty(data_type: DataType) -> Self {
-        Self::from_data(data_type, Buffer::from(&[O::zero()]), Buffer::new(), None)
+        Self::from_data(
+            data_type,
+            Buffer::from(vec![O::zero()]),
+            Buffer::new(),
+            None,
+        )
     }
 
     /// Creates an null [`BinaryArray`], i.e. whose `.null_count() == .len()`.
@@ -90,7 +96,7 @@ impl<O: Offset> BinaryArray<O> {
     /// * The `data_type`'s physical type is not consistent with the offset `O`.
     /// * The last element of `offsets` is different from `values.len()`.
     /// * The validity is not `None` and its length is different from `offsets.len() - 1`.
-    pub fn from_data_unchecked(
+    pub unsafe fn from_data_unchecked(
         data_type: DataType,
         offsets: Buffer<O>,
         values: Buffer<u8>,
