@@ -6,11 +6,7 @@ use crate::{
     datatypes::{DataType, Field},
 };
 
-use super::{
-    display_fmt, new_empty_array,
-    specification::{check_offsets, Offset},
-    Array,
-};
+use super::{debug_fmt, new_empty_array, specification::check_offsets, Array, Offset};
 
 mod ffi;
 mod iterator;
@@ -19,7 +15,7 @@ mod mutable;
 pub use mutable::*;
 
 /// An [`Array`] semantically equivalent to `Vec<Option<Vec<Option<T>>>>` with Arrow's in-memory.
-#[derive(Debug, Clone)]
+#[derive(Clone)]
 pub struct ListArray<O: Offset> {
     data_type: DataType,
     offsets: Buffer<O>,
@@ -31,7 +27,7 @@ impl<O: Offset> ListArray<O> {
     /// Returns a new empty [`ListArray`].
     pub fn new_empty(data_type: DataType) -> Self {
         let values = new_empty_array(Self::get_child_type(&data_type).clone()).into();
-        Self::from_data(data_type, Buffer::from(&[O::zero()]), values, None)
+        Self::from_data(data_type, Buffer::from(vec![O::zero()]), values, None)
     }
 
     /// Returns a new null [`ListArray`].
@@ -241,13 +237,13 @@ impl<O: Offset> Array for ListArray<O> {
     }
 }
 
-impl<O: Offset> std::fmt::Display for ListArray<O> {
+impl<O: Offset> std::fmt::Debug for ListArray<O> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let head = if O::is_large() {
             "LargeListArray"
         } else {
             "ListArray"
         };
-        display_fmt(self.iter(), head, f, true)
+        debug_fmt(self.iter(), head, f, true)
     }
 }
