@@ -18,7 +18,6 @@
 //! Most arrays contain a [`MutableArray`] counterpart that is neither clonable nor slicable, but
 //! can be operated in-place.
 use std::any::Any;
-use std::fmt::Display;
 
 use crate::error::Result;
 use crate::types::{days_ms, months_days_ns};
@@ -27,9 +26,11 @@ use crate::{
     datatypes::DataType,
 };
 
+pub(self) mod physical_binary;
+
 /// A trait representing an immutable Arrow array. Arrow arrays are trait objects
 /// that are infalibly downcasted to concrete types according to the [`Array::data_type`].
-pub trait Array: std::fmt::Debug + Send + Sync {
+pub trait Array: Send + Sync {
     /// Convert to trait object.
     fn as_any(&self) -> &dyn Any;
 
@@ -214,7 +215,7 @@ macro_rules! with_match_primitive_type {(
     }
 })}
 
-impl Display for dyn Array {
+impl std::fmt::Debug for dyn Array + '_ {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         use crate::datatypes::PhysicalType::*;
         match self.data_type().to_physical_type() {
@@ -339,6 +340,14 @@ pub fn clone(array: &dyn Array) -> Box<dyn Array> {
     }
 }
 
+// see https://users.rust-lang.org/t/generic-for-dyn-a-or-box-dyn-a-or-arc-dyn-a/69430/3
+// for details
+impl<'a> AsRef<(dyn Array + 'a)> for dyn Array {
+    fn as_ref(&self) -> &(dyn Array + 'a) {
+        self
+    }
+}
+
 mod binary;
 mod boolean;
 mod dictionary;
@@ -362,6 +371,7 @@ pub mod ord;
 pub use display::get_display;
 pub use equal::equal;
 
+pub use crate::types::Offset;
 pub use binary::{BinaryArray, BinaryValueIter, MutableBinaryArray};
 pub use boolean::{BooleanArray, MutableBooleanArray};
 pub use dictionary::{DictionaryArray, DictionaryKey, MutableDictionaryArray};
@@ -371,7 +381,6 @@ pub use list::{ListArray, MutableListArray};
 pub use map::MapArray;
 pub use null::NullArray;
 pub use primitive::*;
-pub use specification::Offset;
 pub use struct_::StructArray;
 pub use union::UnionArray;
 pub use utf8::{MutableUtf8Array, Utf8Array, Utf8ValuesIter};
@@ -394,31 +403,34 @@ pub trait TryPush<A> {
 }
 
 fn display_helper<T: std::fmt::Display, I: IntoIterator<Item = Option<T>>>(iter: I) -> Vec<String> {
-    let iterator = iter.into_iter();
-    let len = iterator.size_hint().0;
-    if len <= 100 {
-        iterator
-            .map(|x| match x {
-                Some(x) => x.to_string(),
-                None => "".to_string(),
-            })
-            .collect::<Vec<_>>()
+    iter.into_iter()
+        .map(|x| match x {
+            Some(x) => x.to_string(),
+            None => "None".to_string(),
+        })
+        .collect::<Vec<_>>()
+}
+
+fn debug_helper<T: std::fmt::Debug, I: IntoIterator<Item = Option<T>>>(iter: I) -> Vec<String> {
+    iter.into_iter()
+        .map(|x| match x {
+            Some(x) => format!("{:?}", x),
+            None => "None".to_string(),
+        })
+        .collect::<Vec<_>>()
+}
+
+fn debug_fmt<T: std::fmt::Debug, I: IntoIterator<Item = Option<T>>>(
+    iter: I,
+    head: &str,
+    f: &mut std::fmt::Formatter<'_>,
+    new_lines: bool,
+) -> std::fmt::Result {
+    let result = debug_helper(iter);
+    if new_lines {
+        write!(f, "{}[\n{}\n]", head, result.join(",\n"))
     } else {
-        iterator
-            .enumerate()
-            .filter_map(|(i, x)| {
-                if i == 5 {
-                    Some(format!("...({})...", len - 10))
-                } else if i < 5 || i > (len - 5) {
-                    Some(match x {
-                        Some(x) => x.to_string(),
-                        None => "".to_string(),
-                    })
-                } else {
-                    None
-                }
-            })
-            .collect::<Vec<_>>()
+        write!(f, "{}[{}]", head, result.join(", "))
     }
 }
 
