@@ -3,11 +3,10 @@ use std::{iter::FromIterator, sync::Arc};
 use crate::{
     array::{Array, MutableArray, TryExtend, TryPush},
     bitmap::MutableBitmap,
-    buffer::MutableBuffer,
     datatypes::DataType,
     error::{ArrowError, Result},
     trusted_len::TrustedLen,
-    types::{NativeType, NaturalDataType},
+    types::NativeType,
 };
 
 use super::PrimitiveArray;
@@ -17,7 +16,7 @@ use super::PrimitiveArray;
 #[derive(Debug)]
 pub struct MutablePrimitiveArray<T: NativeType> {
     data_type: DataType,
-    values: MutableBuffer<T>,
+    values: Vec<T>,
     validity: Option<MutableBitmap>,
 }
 
@@ -33,13 +32,13 @@ impl<T: NativeType> From<MutablePrimitiveArray<T>> for PrimitiveArray<T> {
     }
 }
 
-impl<T: NativeType + NaturalDataType, P: AsRef<[Option<T>]>> From<P> for MutablePrimitiveArray<T> {
+impl<T: NativeType, P: AsRef<[Option<T>]>> From<P> for MutablePrimitiveArray<T> {
     fn from(slice: P) -> Self {
         Self::from_trusted_len_iter(slice.as_ref().iter().map(|x| x.as_ref()))
     }
 }
 
-impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
+impl<T: NativeType> MutablePrimitiveArray<T> {
     /// Creates a new empty [`MutablePrimitiveArray`].
     pub fn new() -> Self {
         Self::with_capacity(0)
@@ -47,7 +46,7 @@ impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
 
     /// Creates a new [`MutablePrimitiveArray`] with a capacity.
     pub fn with_capacity(capacity: usize) -> Self {
-        Self::with_capacity_from(capacity, T::DATA_TYPE)
+        Self::with_capacity_from(capacity, T::PRIMITIVE.into())
     }
 
     /// Create a [`MutablePrimitiveArray`] out of low-end APIs.
@@ -55,14 +54,10 @@ impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
     /// This function panics iff:
     /// * `data_type` is not supported by the physical type
     /// * The validity is not `None` and its length is different from the `values`'s length
-    pub fn from_data(
-        data_type: DataType,
-        values: MutableBuffer<T>,
-        validity: Option<MutableBitmap>,
-    ) -> Self {
-        if !T::is_valid(&data_type) {
+    pub fn from_data(data_type: DataType, values: Vec<T>, validity: Option<MutableBitmap>) -> Self {
+        if !data_type.to_physical_type().eq_primitive(T::PRIMITIVE) {
             Err(ArrowError::InvalidArgumentError(format!(
-                "Type {} does not support logical type {}",
+                "Type {} does not support logical type {:?}",
                 std::any::type_name::<T>(),
                 data_type
             )))
@@ -79,12 +74,12 @@ impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
     }
 
     /// Extract the low-end APIs from the [`MutablePrimitiveArray`].
-    pub fn into_data(self) -> (DataType, MutableBuffer<T>, Option<MutableBitmap>) {
+    pub fn into_data(self) -> (DataType, Vec<T>, Option<MutableBitmap>) {
         (self.data_type, self.values, self.validity)
     }
 }
 
-impl<T: NativeType + NaturalDataType> Default for MutablePrimitiveArray<T> {
+impl<T: NativeType> Default for MutablePrimitiveArray<T> {
     fn default() -> Self {
         Self::new()
     }
@@ -92,10 +87,10 @@ impl<T: NativeType + NaturalDataType> Default for MutablePrimitiveArray<T> {
 
 impl<T: NativeType> From<DataType> for MutablePrimitiveArray<T> {
     fn from(data_type: DataType) -> Self {
-        assert!(T::is_valid(&data_type));
+        assert!(data_type.to_physical_type().eq_primitive(T::PRIMITIVE));
         Self {
             data_type,
-            values: MutableBuffer::<T>::new(),
+            values: Vec::<T>::new(),
             validity: None,
         }
     }
@@ -104,10 +99,10 @@ impl<T: NativeType> From<DataType> for MutablePrimitiveArray<T> {
 impl<T: NativeType> MutablePrimitiveArray<T> {
     /// Creates a new [`MutablePrimitiveArray`] from a capacity and [`DataType`].
     pub fn with_capacity_from(capacity: usize, data_type: DataType) -> Self {
-        assert!(T::is_valid(&data_type));
+        assert!(data_type.to_physical_type().eq_primitive(T::PRIMITIVE));
         Self {
             data_type,
-            values: MutableBuffer::<T>::with_capacity(capacity),
+            values: Vec::<T>::with_capacity(capacity),
             validity: None,
         }
     }
@@ -142,6 +137,28 @@ impl<T: NativeType> MutablePrimitiveArray<T> {
         }
     }
 
+    /// Extends the [`MutablePrimitiveArray`] with a constant
+    #[inline]
+    pub fn extend_constant(&mut self, additional: usize, value: Option<T>) {
+        if let Some(value) = value {
+            self.values.resize(self.values.len() + additional, value);
+            if let Some(validity) = &mut self.validity {
+                validity.extend_constant(additional, true)
+            }
+        } else {
+            if let Some(validity) = &mut self.validity {
+                validity.extend_constant(additional, false)
+            } else {
+                let mut validity = MutableBitmap::with_capacity(self.values.capacity());
+                validity.extend_constant(self.len(), true);
+                validity.extend_constant(additional, false);
+                self.validity = Some(validity)
+            }
+            self.values
+                .resize(self.values.len() + additional, T::default());
+        }
+    }
+
     /// Extends the [`MutablePrimitiveArray`] from an iterator of trusted len.
     #[inline]
     pub fn extend_trusted_len<P, I>(&mut self, iterator: I)
@@ -191,7 +208,7 @@ impl<T: NativeType> MutablePrimitiveArray<T> {
     where
         I: Iterator<Item = T>,
     {
-        self.values.extend_from_trusted_len_iter_unchecked(iterator);
+        self.values.extend(iterator);
         self.update_all_valid();
     }
 
@@ -250,7 +267,7 @@ impl<T: NativeType> MutablePrimitiveArray<T> {
 /// Accessors
 impl<T: NativeType> MutablePrimitiveArray<T> {
     /// Returns its values.
-    pub fn values(&self) -> &MutableBuffer<T> {
+    pub fn values(&self) -> &Vec<T> {
         &self.values
     }
 
@@ -295,7 +312,7 @@ impl<T: NativeType> MutablePrimitiveArray<T> {
     /// Sets values.
     /// # Panic
     /// Panics iff the values' length is not equal to the existing validity's len.
-    pub fn set_values(&mut self, values: MutableBuffer<T>) {
+    pub fn set_values(&mut self, values: Vec<T>) {
         assert_eq!(values.len(), self.values.len());
         self.values = values;
     }
@@ -371,7 +388,7 @@ impl<T: NativeType> MutableArray for MutablePrimitiveArray<T> {
     }
 }
 
-impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
+impl<T: NativeType> MutablePrimitiveArray<T> {
     /// Creates a [`MutablePrimitiveArray`] from a slice of values.
     pub fn from_slice<P: AsRef<[T]>>(slice: P) -> Self {
         Self::from_trusted_len_values_iter(slice.as_ref().iter().copied())
@@ -390,7 +407,7 @@ impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
         let (validity, values) = trusted_len_unzip(iterator);
 
         Self {
-            data_type: T::DATA_TYPE,
+            data_type: T::PRIMITIVE.into(),
             values,
             validity,
         }
@@ -423,7 +440,7 @@ impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
         let (validity, values) = try_trusted_len_unzip(iterator)?;
 
         Ok(Self {
-            data_type: T::DATA_TYPE,
+            data_type: T::PRIMITIVE.into(),
             values,
             validity,
         })
@@ -442,8 +459,8 @@ impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
     /// Creates a new [`MutablePrimitiveArray`] out an iterator over values
     pub fn from_trusted_len_values_iter<I: TrustedLen<Item = T>>(iter: I) -> Self {
         Self {
-            data_type: T::DATA_TYPE,
-            values: MutableBuffer::<T>::from_trusted_len_iter(iter),
+            data_type: T::PRIMITIVE.into(),
+            values: iter.collect(),
             validity: None,
         }
     }
@@ -454,14 +471,14 @@ impl<T: NativeType + NaturalDataType> MutablePrimitiveArray<T> {
     /// I.e. that `size_hint().1` correctly reports its length.
     pub unsafe fn from_trusted_len_values_iter_unchecked<I: Iterator<Item = T>>(iter: I) -> Self {
         Self {
-            data_type: T::DATA_TYPE,
-            values: MutableBuffer::<T>::from_trusted_len_iter_unchecked(iter),
+            data_type: T::PRIMITIVE.into(),
+            values: iter.collect(),
             validity: None,
         }
     }
 }
 
-impl<T: NativeType + NaturalDataType, Ptr: std::borrow::Borrow<Option<T>>> FromIterator<Ptr>
+impl<T: NativeType, Ptr: std::borrow::Borrow<Option<T>>> FromIterator<Ptr>
     for MutablePrimitiveArray<T>
 {
     fn from_iter<I: IntoIterator<Item = Ptr>>(iter: I) -> Self {
@@ -470,7 +487,7 @@ impl<T: NativeType + NaturalDataType, Ptr: std::borrow::Borrow<Option<T>>> FromI
 
         let mut validity = MutableBitmap::with_capacity(lower);
 
-        let values: MutableBuffer<T> = iter
+        let values: Vec<T> = iter
             .map(|item| {
                 if let Some(a) = item.borrow() {
                     validity.push(true);
@@ -489,14 +506,14 @@ impl<T: NativeType + NaturalDataType, Ptr: std::borrow::Borrow<Option<T>>> FromI
         };
 
         Self {
-            data_type: T::DATA_TYPE,
+            data_type: T::PRIMITIVE.into(),
             values,
             validity,
         }
     }
 }
 
-/// Extends a [`MutableBitmap`] and a [`MutableBuffer`] from an iterator of `Option`.
+/// Extends a [`MutableBitmap`] and a [`Vec`] from an iterator of `Option`.
 /// The first buffer corresponds to a bitmap buffer, the second one
 /// corresponds to a values buffer.
 /// # Safety
@@ -505,7 +522,7 @@ impl<T: NativeType + NaturalDataType, Ptr: std::borrow::Borrow<Option<T>>> FromI
 pub(crate) unsafe fn extend_trusted_len_unzip<I, P, T>(
     iterator: I,
     validity: &mut MutableBitmap,
-    buffer: &mut MutableBuffer<T>,
+    buffer: &mut Vec<T>,
 ) where
     T: NativeType,
     P: std::borrow::Borrow<T>,
@@ -515,36 +532,32 @@ pub(crate) unsafe fn extend_trusted_len_unzip<I, P, T>(
     let additional = upper.expect("trusted_len_unzip requires an upper limit");
 
     validity.reserve(additional);
-    buffer.reserve(additional);
-
-    for item in iterator {
-        let item = if let Some(item) = item {
+    let values = iterator.map(|item| {
+        if let Some(item) = item {
             validity.push_unchecked(true);
             *item.borrow()
         } else {
             validity.push_unchecked(false);
             T::default()
-        };
-        buffer.push_unchecked(item);
-    }
+        }
+    });
+    buffer.extend(values);
 }
 
-/// Creates a [`MutableBitmap`] and a [`MutableBuffer`] from an iterator of `Option`.
+/// Creates a [`MutableBitmap`] and a [`Vec`] from an iterator of `Option`.
 /// The first buffer corresponds to a bitmap buffer, the second one
 /// corresponds to a values buffer.
 /// # Safety
 /// The caller must ensure that `iterator` is `TrustedLen`.
 #[inline]
-pub(crate) unsafe fn trusted_len_unzip<I, P, T>(
-    iterator: I,
-) -> (Option<MutableBitmap>, MutableBuffer<T>)
+pub(crate) unsafe fn trusted_len_unzip<I, P, T>(iterator: I) -> (Option<MutableBitmap>, Vec<T>)
 where
     T: NativeType,
     P: std::borrow::Borrow<T>,
     I: Iterator<Item = Option<P>>,
 {
     let mut validity = MutableBitmap::new();
-    let mut buffer = MutableBuffer::<T>::new();
+    let mut buffer = Vec::<T>::new();
 
     extend_trusted_len_unzip(iterator, &mut validity, &mut buffer);
 
@@ -562,7 +575,7 @@ where
 #[inline]
 pub(crate) unsafe fn try_trusted_len_unzip<E, I, P, T>(
     iterator: I,
-) -> std::result::Result<(Option<MutableBitmap>, MutableBuffer<T>), E>
+) -> std::result::Result<(Option<MutableBitmap>, Vec<T>), E>
 where
     T: NativeType,
     P: std::borrow::Borrow<T>,
@@ -572,7 +585,7 @@ where
     let len = upper.expect("trusted_len_unzip requires an upper limit");
 
     let mut null = MutableBitmap::with_capacity(len);
-    let mut buffer = MutableBuffer::<T>::with_capacity(len);
+    let mut buffer = Vec::<T>::with_capacity(len);
 
     let mut dst = buffer.as_mut_ptr();
     for item in iterator {
