@@ -17,10 +17,10 @@ pub mod decimal;
 pub mod time;
 
 use crate::{
-    array::Array,
+    array::{Array, DictionaryArray, PrimitiveArray},
     bitmap::Bitmap,
     datatypes::{DataType, IntervalUnit, TimeUnit},
-    types::NativeType,
+    scalar::{PrimitiveScalar, Scalar},
 };
 
 // Macro to evaluate match branch in arithmetic function.
@@ -101,6 +101,99 @@ macro_rules! arith {
     }};
 }
 
+// Macro to evaluate match branch in arithmetic function.
+macro_rules! primitive_scalar {
+    ($lhs:expr, $rhs:expr, $op:tt, $type:ty) => {{
+        let lhs = $lhs
+            .as_any()
+            .downcast_ref::<PrimitiveArray<$type>>()
+            .unwrap();
+        let rhs = $rhs
+            .as_any()
+            .downcast_ref::<PrimitiveScalar<$type>>()
+            .unwrap();
+
+        let rhs = if let Some(rhs) = rhs.value() {
+            rhs
+        } else {
+            return Box::new(PrimitiveArray::<$type>::new_null(
+                lhs.data_type().clone(),
+                lhs.len(),
+            )) as Box<dyn Array>;
+        };
+
+        let result = basic::$op::<$type>(lhs, &rhs);
+        Box::new(result) as Box<dyn Array>
+    }};
+}
+
+// Macro to create a `match` statement with dynamic dispatch to functions based on
+// the array's logical types
+macro_rules! arith_scalar {
+    ($lhs:expr, $rhs:expr, $op:tt $(, decimal = $op_decimal:tt )? $(, duration = $op_duration:tt )? $(, interval = $op_interval:tt )? $(, timestamp = $op_timestamp:tt )?) => {{
+        let lhs = $lhs;
+        let rhs = $rhs;
+        use DataType::*;
+        match (lhs.data_type(), rhs.data_type()) {
+            (Int8, Int8) => primitive_scalar!(lhs, rhs, $op, i8),
+            (Int16, Int16) => primitive_scalar!(lhs, rhs, $op, i16),
+            (Int32, Int32) => primitive_scalar!(lhs, rhs, $op, i32),
+            (Int64, Int64) | (Duration(_), Duration(_)) => {
+                primitive_scalar!(lhs, rhs, $op, i64)
+            }
+            (UInt8, UInt8) => primitive_scalar!(lhs, rhs, $op, u8),
+            (UInt16, UInt16) => primitive_scalar!(lhs, rhs, $op, u16),
+            (UInt32, UInt32) => primitive_scalar!(lhs, rhs, $op, u32),
+            (UInt64, UInt64) => primitive_scalar!(lhs, rhs, $op, u64),
+            (Float32, Float32) => primitive_scalar!(lhs, rhs, $op, f32),
+            (Float64, Float64) => primitive_scalar!(lhs, rhs, $op, f64),
+            $ (
+            (Decimal(_, _), Decimal(_, _)) => {
+                let lhs = lhs.as_any().downcast_ref().unwrap();
+                let rhs = rhs.as_any().downcast_ref().unwrap();
+                Box::new(decimal::$op_decimal(lhs, rhs)) as Box<dyn Array>
+            }
+            )?
+            $ (
+            (Time32(TimeUnit::Second), Duration(_))
+            | (Time32(TimeUnit::Millisecond), Duration(_))
+            | (Date32, Duration(_)) => {
+                let lhs = lhs.as_any().downcast_ref().unwrap();
+                let rhs = rhs.as_any().downcast_ref().unwrap();
+                Box::new(time::$op_duration::<i32>(lhs, rhs)) as Box<dyn Array>
+            }
+            (Time64(TimeUnit::Microsecond), Duration(_))
+            | (Time64(TimeUnit::Nanosecond), Duration(_))
+            | (Date64, Duration(_))
+            | (Timestamp(_, _), Duration(_)) => {
+                let lhs = lhs.as_any().downcast_ref().unwrap();
+                let rhs = rhs.as_any().downcast_ref().unwrap();
+                Box::new(time::$op_duration::<i64>(lhs, rhs)) as Box<dyn Array>
+            }
+            )?
+            $ (
+            (Timestamp(_, _), Interval(IntervalUnit::MonthDayNano)) => {
+                let lhs = lhs.as_any().downcast_ref().unwrap();
+                let rhs = rhs.as_any().downcast_ref().unwrap();
+                time::$op_interval(lhs, rhs).map(|x| Box::new(x) as Box<dyn Array>).unwrap()
+            }
+            )?
+            $ (
+            (Timestamp(_, None), Timestamp(_, None)) => {
+                let lhs = lhs.as_any().downcast_ref().unwrap();
+                let rhs = rhs.as_any().downcast_ref().unwrap();
+                time::$op_timestamp(lhs, rhs).map(|x| Box::new(x) as Box<dyn Array>).unwrap()
+            }
+            )?
+            _ => todo!(
+                "Addition of {:?} with {:?} is not supported",
+                lhs.data_type(),
+                rhs.data_type()
+            ),
+        }
+    }};
+}
+
 /// Adds two [`Array`]s.
 /// # Panic
 /// This function panics iff
@@ -117,6 +210,22 @@ pub fn add(lhs: &dyn Array, rhs: &dyn Array) -> Box<dyn Array> {
     )
 }
 
+/// Adds an [`Array`] and a [`Scalar`].
+/// # Panic
+/// This function panics iff
+/// * the opertion is not supported for the logical types (use [`can_add`] to check)
+/// * the arrays have a different length
+/// * one of the arrays is a timestamp with timezone and the timezone is not valid.
+pub fn add_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> Box<dyn Array> {
+    arith_scalar!(
+        lhs,
+        rhs,
+        add_scalar,
+        duration = add_duration_scalar,
+        interval = add_interval_scalar
+    )
+}
+
 /// Returns whether two [`DataType`]s can be added by [`add`].
 pub fn can_add(lhs: &DataType, rhs: &DataType) -> bool {
     use DataType::*;
@@ -162,6 +271,22 @@ pub fn sub(lhs: &dyn Array, rhs: &dyn Array) -> Box<dyn Array> {
     )
 }
 
+/// Adds an [`Array`] and a [`Scalar`].
+/// # Panic
+/// This function panics iff
+/// * the opertion is not supported for the logical types (use [`can_sub`] to check)
+/// * the arrays have a different length
+/// * one of the arrays is a timestamp with timezone and the timezone is not valid.
+pub fn sub_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> Box<dyn Array> {
+    arith_scalar!(
+        lhs,
+        rhs,
+        sub_scalar,
+        duration = sub_duration_scalar,
+        timestamp = sub_timestamps_scalar
+    )
+}
+
 /// Returns whether two [`DataType`]s can be subtracted by [`sub`].
 pub fn can_sub(lhs: &DataType, rhs: &DataType) -> bool {
     use DataType::*;
@@ -199,6 +324,14 @@ pub fn mul(lhs: &dyn Array, rhs: &dyn Array) -> Box<dyn Array> {
     arith!(lhs, rhs, mul, decimal = mul)
 }
 
+/// Multiply an [`Array`] with a [`Scalar`].
+/// # Panic
+/// This function panics iff
+/// * the opertion is not supported for the logical types (use [`can_mul`] to check)
+pub fn mul_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> Box<dyn Array> {
+    arith_scalar!(lhs, rhs, mul_scalar, decimal = mul_scalar)
+}
+
 /// Returns whether two [`DataType`]s can be multiplied by [`mul`].
 pub fn can_mul(lhs: &DataType, rhs: &DataType) -> bool {
     use DataType::*;
@@ -227,6 +360,14 @@ pub fn div(lhs: &dyn Array, rhs: &dyn Array) -> Box<dyn Array> {
     arith!(lhs, rhs, div, decimal = div)
 }
 
+/// Divide an [`Array`] with a [`Scalar`].
+/// # Panic
+/// This function panics iff
+/// * the opertion is not supported for the logical types (use [`can_div`] to check)
+pub fn div_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> Box<dyn Array> {
+    arith_scalar!(lhs, rhs, div_scalar, decimal = div_scalar)
+}
+
 /// Returns whether two [`DataType`]s can be divided by [`div`].
 pub fn can_div(lhs: &DataType, rhs: &DataType) -> bool {
     can_mul(lhs, rhs)
@@ -259,6 +400,72 @@ pub fn can_rem(lhs: &DataType, rhs: &DataType) -> bool {
     )
 }
 
+macro_rules! with_match_negatable {(
+    $key_type:expr, | $_:tt $T:ident | $($body:tt)*
+) => ({
+    macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    use crate::datatypes::PrimitiveType::*;
+    use crate::types::{days_ms, months_days_ns};
+    match $key_type {
+        Int8 => __with_ty__! { i8 },
+        Int16 => __with_ty__! { i16 },
+        Int32 => __with_ty__! { i32 },
+        Int64 => __with_ty__! { i64 },
+        Int128 => __with_ty__! { i128 },
+        DaysMs => __with_ty__! { days_ms },
+        MonthDayNano => __with_ty__! { months_days_ns },
+        UInt8 | UInt16 | UInt32 | UInt64=> todo!(),
+        Float32 => __with_ty__! { f32 },
+        Float64 => __with_ty__! { f64 },
+    }
+})}
+
+/// Negates an [`Array`].
+/// # Panic
+/// This function panics iff either
+/// * the opertion is not supported for the logical type (use [`can_neg`] to check)
+/// * the operation overflows
+pub fn neg(array: &dyn Array) -> Box<dyn Array> {
+    use crate::datatypes::PhysicalType::*;
+    match array.data_type().to_physical_type() {
+        Primitive(primitive) => with_match_negatable!(primitive, |$T| {
+            let array = array.as_any().downcast_ref().unwrap();
+
+            let result = basic::negate::<$T>(array);
+            Box::new(result) as Box<dyn Array>
+        }),
+        Dictionary(key) => match_integer_type!(key, |$T| {
+            let array = array.as_any().downcast_ref::<DictionaryArray<$T>>().unwrap();
+
+            let values = neg(array.values().as_ref()).into();
+
+            Box::new(DictionaryArray::<$T>::from_data(array.keys().clone(), values)) as Box<dyn Array>
+        }),
+        _ => todo!(),
+    }
+}
+
+/// Whether [`neg`] is supported for a given [`DataType`]
+pub fn can_neg(data_type: &DataType) -> bool {
+    if let DataType::Dictionary(_, values, _) = data_type.to_logical_type() {
+        return can_neg(values.as_ref());
+    }
+
+    use crate::datatypes::PhysicalType::*;
+    use crate::datatypes::PrimitiveType::*;
+    matches!(
+        data_type.to_physical_type(),
+        Primitive(Int8)
+            | Primitive(Int16)
+            | Primitive(Int32)
+            | Primitive(Int64)
+            | Primitive(Float64)
+            | Primitive(Float32)
+            | Primitive(DaysMs)
+            | Primitive(MonthDayNano)
+    )
+}
+
 /// Defines basic addition operation for primitive arrays
 pub trait ArrayAdd<Rhs>: Sized {
     /// Adds itself to `rhs`
@@ -372,19 +579,3 @@ pub trait ArrayCheckedRem<Rhs>: Sized {
     /// checked remainder
     fn checked_rem(&self, rhs: &Rhs) -> Self;
 }
-
-/// Trait describing a [`NativeType`] whose semantics of arithmetic in Arrow equals
-/// the semantics in Rust.
-/// A counter example is `i128`, that in arrow represents a decimal while in rust represents
-/// a signed integer.
-pub trait NativeArithmetics: NativeType {}
-impl NativeArithmetics for u8 {}
-impl NativeArithmetics for u16 {}
-impl NativeArithmetics for u32 {}
-impl NativeArithmetics for u64 {}
-impl NativeArithmetics for i8 {}
-impl NativeArithmetics for i16 {}
-impl NativeArithmetics for i32 {}
-impl NativeArithmetics for i64 {}
-impl NativeArithmetics for f32 {}
-impl NativeArithmetics for f64 {}
