@@ -4,7 +4,6 @@ use crate::compute::take;
 use crate::error::{ArrowError, Result};
 use crate::{
     array::{ord, Array, PrimitiveArray},
-    buffer::MutableBuffer,
     types::Index,
 };
 
@@ -40,7 +39,7 @@ pub struct SortColumn<'a> {
 /// let int64 = Int64Array::from(&[None, Some(-2), Some(89), Some(-64), Some(101)]);
 /// let utf8 = Utf8Array::<i32>::from(&vec![Some("hello"), Some("world"), Some(","), Some("foobar"), Some("!")]);
 ///
-/// let sorted_columns = lexsort::<i32>(&vec![
+/// let sorted_chunk = lexsort::<i32>(&vec![
 ///     SortColumn {
 ///         values: &int64,
 ///         options: None,
@@ -54,7 +53,7 @@ pub struct SortColumn<'a> {
 ///     },
 /// ], None).unwrap();
 ///
-/// let sorted = sorted_columns[0].as_any().downcast_ref::<Int64Array>().unwrap();
+/// let sorted = sorted_chunk[0].as_any().downcast_ref::<Int64Array>().unwrap();
 /// assert_eq!(sorted.value(1), -64);
 /// assert!(sorted.is_null(0));
 /// ```
@@ -168,12 +167,7 @@ pub fn lexsort_to_indices<I: Index>(
         Ordering::Equal
     };
 
-    // Safety: `0..row_count` is TrustedLen
-    let mut values = unsafe {
-        MutableBuffer::from_trusted_len_iter_unchecked(
-            (0..row_count).map(|x| I::from_usize(x).unwrap()),
-        )
-    };
+    let mut values = I::range(0, row_count).unwrap().collect::<Vec<_>>();
 
     if let Some(limit) = limit {
         let limit = limit.min(row_count);
@@ -185,8 +179,9 @@ pub fn lexsort_to_indices<I: Index>(
         values.sort_unstable_by(lex_comparator);
     }
 
+    let data_type = I::PRIMITIVE.into();
     Ok(PrimitiveArray::<I>::from_data(
-        I::DATA_TYPE,
+        data_type,
         values.into(),
         None,
     ))
