@@ -1,5 +1,7 @@
 #![deny(missing_docs)]
+#![forbid(unsafe_code)]
 //! Contains all metadata, such as [`PhysicalType`], [`DataType`], [`Field`] and [`Schema`].
+
 mod field;
 mod physical_type;
 mod schema;
@@ -8,8 +10,18 @@ pub use field::Field;
 pub use physical_type::*;
 pub use schema::Schema;
 
-/// The set of supported logical types.
-/// Each variant uniquely identifies a logical type, which define specific semantics to the data (e.g. how it should be represented).
+use std::collections::BTreeMap;
+use std::sync::Arc;
+
+/// typedef for [BTreeMap<String, String>] denoting [`Field`]'s and [`Schema`]'s metadata.
+pub type Metadata = BTreeMap<String, String>;
+/// typedef fpr [Option<(String, Option<String>)>] descr
+pub(crate) type Extension = Option<(String, Option<String>)>;
+
+/// The set of supported logical types in this crate.
+///
+/// Each variant uniquely identifies a logical type, which define specific semantics to the data
+/// (e.g. how it should be represented).
 /// Each variant has a corresponding [`PhysicalType`], obtained via [`DataType::to_physical_type`],
 /// which declares the in-memory representation of data.
 /// The [`DataType::Extension`] is special in that it augments a [`DataType`] with metadata to support custom types.
@@ -20,74 +32,77 @@ pub enum DataType {
     Null,
     /// `true` and `false`.
     Boolean,
-    /// A signed 8-bit integer.
+    /// An [`i8`]
     Int8,
-    /// A signed 16-bit integer.
+    /// An [`i16`]
     Int16,
-    /// A signed 32-bit integer.
+    /// An [`i32`]
     Int32,
-    /// A signed 64-bit integer.
+    /// An [`i64`]
     Int64,
-    /// An unsigned 8-bit integer.
+    /// An [`u8`]
     UInt8,
-    /// An unsigned 16-bit integer.
+    /// An [`u16`]
     UInt16,
-    /// An unsigned 32-bit integer.
+    /// An [`u32`]
     UInt32,
-    /// An unsigned 64-bit integer.
+    /// An [`u64`]
     UInt64,
-    /// A 16-bit floating point number.
+    /// An 16-bit float
     Float16,
-    /// A 32-bit floating point number.
+    /// A [`f32`]
     Float32,
-    /// A 64-bit floating point number.
+    /// A [`f64`]
     Float64,
-    /// A timestamp with an optional timezone.
+    /// A [`i64`] representing a timestamp measured in [`TimeUnit`] with an optional timezone.
     ///
     /// Time is measured as a Unix epoch, counting the seconds from
     /// 00:00:00.000 on 1 January 1970, excluding leap seconds,
-    /// as a 64-bit integer.
+    /// as a 64-bit signed integer.
     ///
     /// The time zone is a string indicating the name of a time zone, one of:
     ///
     /// * As used in the Olson time zone database (the "tz database" or
     ///   "tzdata"), such as "America/New_York"
     /// * An absolute time zone offset of the form +XX:XX or -XX:XX, such as +07:30
+    /// When the timezone is not specified, the timestamp is considered to have no timezone
+    /// and is represented _as is_
     Timestamp(TimeUnit, Option<String>),
-    /// A 32-bit date representing the elapsed time since UNIX epoch (1970-01-01)
-    /// in days (32 bits).
+    /// An [`i32`] representing the elapsed time since UNIX epoch (1970-01-01)
+    /// in days.
     Date32,
-    /// A 64-bit date representing the elapsed time since UNIX epoch (1970-01-01)
-    /// in milliseconds (64 bits). Values are evenly divisible by 86400000.
+    /// An [`i64`] representing the elapsed time since UNIX epoch (1970-01-01)
+    /// in milliseconds. Values are evenly divisible by 86400000.
     Date64,
     /// A 32-bit time representing the elapsed time since midnight in the unit of `TimeUnit`.
+    /// Only [`TimeUnit::Second`] and [`TimeUnit::Millisecond`] are supported on this variant.
     Time32(TimeUnit),
     /// A 64-bit time representing the elapsed time since midnight in the unit of `TimeUnit`.
+    /// Only [`TimeUnit::Microsecond`] and [`TimeUnit::Nanosecond`] are supported on this variant.
     Time64(TimeUnit),
-    /// Measure of elapsed time in either seconds, milliseconds, microseconds or nanoseconds.
+    /// Measure of elapsed time. This elapsed time is a physical duration (i.e. 1s as defined in S.I.)
     Duration(TimeUnit),
-    /// A "calendar" interval which models types that don't necessarily
-    /// have a precise duration without the context of a base timestamp (e.g.
-    /// days can differ in length during day light savings time transitions).
+    /// A "calendar" interval modeling elapsed time that takes into account calendar shifts.
+    /// For example an interval of 1 day may represent more than 24 hours.
     Interval(IntervalUnit),
-    /// Opaque binary data of variable length.
+    /// Opaque binary data of variable length whose offsets are represented as [`i32`].
     Binary,
     /// Opaque binary data of fixed size.
     /// Enum parameter specifies the number of bytes per value.
     FixedSizeBinary(usize),
-    /// Opaque binary data of variable length and 64-bit offsets.
+    /// Opaque binary data of variable length whose offsets are represented as [`i64`].
     LargeBinary,
-    /// A variable-length string in Unicode with UTF-8 encoding.
+    /// A variable-length UTF-8 encoded string whose offsets are represented as [`i32`].
     Utf8,
-    /// A variable-length string in Unicode with UFT-8 encoding and 64-bit offsets.
+    /// A variable-length UTF-8 encoded string whose offsets are represented as [`i64`].
     LargeUtf8,
-    /// A list of some logical data type with variable length.
+    /// A list of some logical data type whose offsets are represented as [`i32`].
     List(Box<Field>),
-    /// A list of some logical data type with fixed length.
+    /// A list of some logical data type with a fixed number of elements.
     FixedSizeList(Box<Field>, usize),
-    /// A list of some logical data type with variable length and 64-bit offsets.
+    /// A list of some logical data type whose offsets are represented as [`i64`].
     LargeList(Box<Field>),
-    /// A nested datatype that contains a number of sub-fields.
+    /// A nested [`DataType`] with a given number of [`Field`]s.
     Struct(Vec<Field>),
     /// A nested datatype that can represent slots of differing types.
     /// Third argument represents mode
@@ -128,7 +143,9 @@ pub enum DataType {
     ///
     /// This type mostly used to represent low cardinality string
     /// arrays or a limited set of primitive types as integers.
-    Dictionary(IntegerType, Box<DataType>),
+    ///
+    /// The `bool` value indicates the `Dictionary` is sorted if set to `true`.
+    Dictionary(IntegerType, Box<DataType>, bool),
     /// Decimal value with precision and scale
     /// precision is the number of digits in the number and
     /// scale is the number of decimal places.
@@ -138,12 +155,6 @@ pub enum DataType {
     Extension(String, Box<DataType>, Option<String>),
 }
 
-impl std::fmt::Display for DataType {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        write!(f, "{:?}", self)
-    }
-}
-
 /// Mode of [`DataType::Union`]
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum UnionMode {
@@ -191,45 +202,16 @@ pub enum TimeUnit {
 /// Interval units defined in Arrow
 #[derive(Debug, Clone, Copy, PartialEq, Eq, Hash)]
 pub enum IntervalUnit {
-    /// Indicates the number of elapsed whole months, stored as 4-byte integers.
+    /// The number of elapsed whole months.
     YearMonth,
-    /// Indicates the number of elapsed days and milliseconds,
-    /// stored as 2 contiguous 32-bit integers (8-bytes in total).
+    /// The number of elapsed days and milliseconds,
+    /// stored as 2 contiguous `i32`
     DayTime,
-    /// The values are stored contiguously in 16 byte blocks. Months and
-    /// days are encoded as 32 bit integers and nanoseconds is encoded as a
-    /// 64 bit integer. All integers are signed. Each field is independent
-    /// (e.g. there is no constraint that nanoseconds have the same sign
-    /// as days or that the quantitiy of nanoseconds represents less
-    /// then a day's worth of time).
+    /// The number of elapsed months (i32), days (i32) and nanoseconds (i64).
     MonthDayNano,
 }
 
 impl DataType {
-    /// Compares the datatype with another, ignoring nested field names
-    /// and metadata.
-    pub(crate) fn equals_datatype(&self, other: &DataType) -> bool {
-        match (&self, other) {
-            (DataType::List(a), DataType::List(b))
-            | (DataType::LargeList(a), DataType::LargeList(b)) => {
-                a.is_nullable() == b.is_nullable() && a.data_type().equals_datatype(b.data_type())
-            }
-            (DataType::FixedSizeList(a, a_size), DataType::FixedSizeList(b, b_size)) => {
-                a_size == b_size
-                    && a.is_nullable() == b.is_nullable()
-                    && a.data_type().equals_datatype(b.data_type())
-            }
-            (DataType::Struct(a), DataType::Struct(b)) => {
-                a.len() == b.len()
-                    && a.iter().zip(b).all(|(a, b)| {
-                        a.is_nullable() == b.is_nullable()
-                            && a.data_type().equals_datatype(b.data_type())
-                    })
-            }
-            _ => self == other,
-        }
-    }
-
     /// the [`PhysicalType`] of this [`DataType`].
     pub fn to_physical_type(&self) -> PhysicalType {
         use DataType::*;
@@ -267,7 +249,7 @@ impl DataType {
             Struct(_) => PhysicalType::Struct,
             Union(_, _, _) => PhysicalType::Union,
             Map(_, _) => PhysicalType::Map,
-            Dictionary(key, _) => PhysicalType::Dictionary(*key),
+            Dictionary(key, _, _) => PhysicalType::Dictionary(*key),
             Extension(_, key, _) => key.to_physical_type(),
         }
     }
@@ -299,27 +281,34 @@ impl From<IntegerType> for DataType {
     }
 }
 
-// backward compatibility
-use std::collections::BTreeMap;
-use std::sync::Arc;
+impl From<PrimitiveType> for DataType {
+    fn from(item: PrimitiveType) -> Self {
+        match item {
+            PrimitiveType::Int8 => DataType::Int8,
+            PrimitiveType::Int16 => DataType::Int16,
+            PrimitiveType::Int32 => DataType::Int32,
+            PrimitiveType::Int64 => DataType::Int64,
+            PrimitiveType::UInt8 => DataType::UInt8,
+            PrimitiveType::UInt16 => DataType::UInt16,
+            PrimitiveType::UInt32 => DataType::UInt32,
+            PrimitiveType::UInt64 => DataType::UInt64,
+            PrimitiveType::Int128 => DataType::Decimal(32, 32),
+            PrimitiveType::Float32 => DataType::Float32,
+            PrimitiveType::Float64 => DataType::Float64,
+            PrimitiveType::DaysMs => DataType::Interval(IntervalUnit::DayTime),
+            PrimitiveType::MonthDayNano => DataType::Interval(IntervalUnit::MonthDayNano),
+        }
+    }
+}
 
 /// typedef for [`Arc<Schema>`].
 pub type SchemaRef = Arc<Schema>;
 
-/// typedef for [Option<BTreeMap<String, String>>].
-pub type Metadata = Option<BTreeMap<String, String>>;
-/// typedef fpr [Option<(String, Option<String>)>].
-pub type Extension = Option<(String, Option<String>)>;
-
 /// support get extension for metadata
-pub fn get_extension(metadata: &Option<BTreeMap<String, String>>) -> Extension {
-    if let Some(metadata) = metadata {
-        if let Some(name) = metadata.get("ARROW:extension:name") {
-            let metadata = metadata.get("ARROW:extension:metadata").cloned();
-            Some((name.clone(), metadata))
-        } else {
-            None
-        }
+pub fn get_extension(metadata: &Metadata) -> Extension {
+    if let Some(name) = metadata.get("ARROW:extension:name") {
+        let metadata = metadata.get("ARROW:extension:metadata").cloned();
+        Some((name.clone(), metadata))
     } else {
         None
     }
