@@ -56,9 +56,9 @@ impl Ffi_ArrowSchema {
     /// creates a new [Ffi_ArrowSchema]
     pub(crate) fn new(field: &Field) -> Self {
         let format = to_format(field.data_type());
-        let name = field.name().clone();
+        let name = field.name.clone();
 
-        let mut flags = field.is_nullable() as i64 * 2;
+        let mut flags = field.is_nullable as i64 * 2;
 
         // allocate (and hold) the children
         let children_vec = match field.data_type() {
@@ -92,8 +92,8 @@ impl Ffi_ArrowSchema {
             .collect::<Box<_>>();
         let n_children = children_ptr.len() as i64;
 
-        let dictionary = if let DataType::Dictionary(_, values) = field.data_type() {
-            flags += field.dict_is_ordered().unwrap_or_default() as i64;
+        let dictionary = if let DataType::Dictionary(_, values, is_ordered) = field.data_type() {
+            flags += *is_ordered as i64;
             // we do not store field info in the dict values, so can't recover it all :(
             let field = Field::new("", values.as_ref().clone(), true);
             Some(Box::new(Ffi_ArrowSchema::new(&field)))
@@ -101,11 +101,11 @@ impl Ffi_ArrowSchema {
             None
         };
 
-        let metadata = field.metadata();
+        let metadata = &field.metadata;
 
         let metadata = if let DataType::Extension(name, _, extension_metadata) = field.data_type() {
             // append extension information.
-            let mut metadata = metadata.clone().unwrap_or_default();
+            let mut metadata = metadata.clone();
 
             // metadata
             if let Some(extension_metadata) = extension_metadata {
@@ -118,8 +118,10 @@ impl Ffi_ArrowSchema {
             metadata.insert("ARROW:extension:name".to_string(), name.clone());
 
             Some(metadata_to_bytes(&metadata))
+        } else if !metadata.is_empty() {
+            Some(metadata_to_bytes(metadata))
         } else {
-            metadata.as_ref().map(metadata_to_bytes)
+            None
         };
 
         let name = CString::new(name).unwrap();
@@ -214,7 +216,8 @@ pub(crate) unsafe fn to_field(schema: &Ffi_ArrowSchema) -> Result<Field> {
     let data_type = if let Some(dictionary) = dictionary {
         let indices = to_integer_type(schema.format())?;
         let values = to_field(dictionary)?;
-        DataType::Dictionary(indices, Box::new(values.data_type().clone()))
+        let is_ordered = schema.flags & 1 == 1;
+        DataType::Dictionary(indices, Box::new(values.data_type().clone()), is_ordered)
     } else {
         to_data_type(schema)?
     };
@@ -226,9 +229,7 @@ pub(crate) unsafe fn to_field(schema: &Ffi_ArrowSchema) -> Result<Field> {
         data_type
     };
 
-    let mut field = Field::new(schema.name(), data_type, schema.nullable());
-    field.set_metadata(metadata);
-    Ok(field)
+    Ok(Field::new(schema.name(), data_type, schema.nullable()).with_metadata(metadata))
 }
 
 fn to_integer_type(format: &str) -> Result<IntegerType> {
@@ -449,7 +450,7 @@ fn to_format(data_type: &DataType) -> String {
             r
         }
         DataType::Map(_, _) => "+m".to_string(),
-        DataType::Dictionary(index, _) => to_format(&(*index).into()),
+        DataType::Dictionary(index, _, _) => to_format(&(*index).into()),
         DataType::Extension(_, inner, _) => to_format(inner.as_ref()),
     }
 }
@@ -493,7 +494,7 @@ unsafe fn read_bytes(ptr: *const u8, len: usize) -> &'static str {
 unsafe fn metadata_from_bytes(data: *const ::std::os::raw::c_char) -> (Metadata, Extension) {
     let mut data = data as *const u8; // u8 = i8
     if data.is_null() {
-        return (None, None);
+        return (Metadata::default(), None);
     };
     let len = read_ne_i32(data);
     data = data.add(4);
@@ -523,5 +524,5 @@ unsafe fn metadata_from_bytes(data: *const ::std::os::raw::c_char) -> (Metadata,
         };
     }
     let extension = extension_name.map(|name| (name, extension_metadata));
-    (Some(result), extension)
+    (result, extension)
 }
