@@ -10,9 +10,8 @@
 //! using the more integrated approach that is exposed in this module.
 //!
 //! [Arrow's IPC protocol](https://arrow.apache.org/docs/format/Columnar.html#serialization-and-interprocess-communication-ipc)
-//! allows only [`RecordBatch`](crate::record_batch::RecordBatch)es or
-//! [`DictionaryBatch`](gen::Message::DictionaryBatch) to be passed
-//! around due to its reliance on a pre-defined data scheme. This limitation
+//! allows only batch or dictionary columns to be passed
+//! around due to its reliance on a pre-defined data scheme. This constraint
 //! provides a large performance gain because serialized data will always have a
 //! known structutre, i.e. the same fields and datatypes, with the only variance
 //! being the number of rows and the actual data inside the Batch. This dramatically
@@ -34,29 +33,28 @@
 //! # use std::fs::File;
 //! # use std::sync::Arc;
 //! # use arrow2::datatypes::{Field, Schema, DataType};
-//! # use arrow2::array::Int32Array;
-//! # use arrow2::record_batch::RecordBatch;
+//! # use arrow2::array::{Int32Array, Array};
+//! # use arrow2::chunk::Chunk;
 //! # use arrow2::error::ArrowError;
 //! // Setup the writer
 //! let path = "example.arrow".to_string();
 //! let mut file = File::create(&path)?;
 //! let x_coord = Field::new("x", DataType::Int32, false);
 //! let y_coord = Field::new("y", DataType::Int32, false);
-//! let schema = Schema::new(vec![x_coord, y_coord]);
+//! let schema = Schema::from(vec![x_coord, y_coord]);
 //! let options = WriteOptions {compression: None};
-//! let mut writer = FileWriter::try_new(file, &schema, options)?;
+//! let mut writer = FileWriter::try_new(file, &schema, None, options)?;
 //!
 //! // Setup the data
 //! let x_data = Int32Array::from_slice([-1i32, 1]);
 //! let y_data = Int32Array::from_slice([1i32, -1]);
-//! let batch = RecordBatch::try_new(
-//!        Arc::new(schema),
-//!         vec![Arc::new(x_data), Arc::new(y_data)]
-//!    )?;
+//! let chunk = Chunk::try_new(
+//!     vec![Arc::new(x_data) as Arc<dyn Array>, Arc::new(y_data)]
+//! )?;
 //!
 //! // Write the messages and finalize the stream
 //! for _ in 0..5 {
-//!     writer.write(&batch);
+//!     writer.write(&chunk, None);
 //! }
 //! writer.finish();
 //!
@@ -78,13 +76,35 @@
 //! [2](https://github.com/jorgecarleitao/arrow2/blob/main/examples/ipc_file_write.rs),
 //! [3](https://github.com/jorgecarleitao/arrow2/tree/main/examples/ipc_pyarrow)).
 
+use crate::error::ArrowError;
+
 mod compression;
-mod convert;
 mod endianess;
 
-pub use convert::fb_to_schema;
 pub mod read;
 pub mod write;
 
 const ARROW_MAGIC: [u8; 6] = [b'A', b'R', b'R', b'O', b'W', b'1'];
 const CONTINUATION_MARKER: [u8; 4] = [0xff; 4];
+
+/// Struct containing `dictionary_id` and nested `IpcField`, allowing users
+/// to specify the dictionary ids of the IPC fields when writing to IPC.
+#[derive(Debug, Clone, PartialEq, Default)]
+pub struct IpcField {
+    // optional children
+    pub fields: Vec<IpcField>,
+    // dictionary id
+    pub dictionary_id: Option<i64>,
+}
+
+#[derive(Debug, Clone, PartialEq)]
+pub struct IpcSchema {
+    pub fields: Vec<IpcField>,
+    pub is_little_endian: bool,
+}
+
+impl From<arrow_format::ipc::planus::Error> for ArrowError {
+    fn from(error: arrow_format::ipc::planus::Error) -> Self {
+        ArrowError::OutOfSpec(error.to_string())
+    }
+}
