@@ -1,31 +1,29 @@
 use std::collections::VecDeque;
-use std::convert::TryInto;
 use std::io::{Read, Seek};
 
-use arrow_format::ipc;
-
 use crate::array::{Offset, Utf8Array};
 use crate::buffer::Buffer;
 use crate::datatypes::DataType;
-use crate::error::Result;
-use crate::types::NativeType;
+use crate::error::{ArrowError, Result};
 
-use super::super::deserialize::Node;
 use super::super::read_basic::*;
+use super::super::{Compression, IpcBuffer, Node};
 
 pub fn read_utf8<O: Offset, R: Read + Seek>(
     field_nodes: &mut VecDeque<Node>,
     data_type: DataType,
-    buffers: &mut VecDeque<&ipc::Schema::Buffer>,
+    buffers: &mut VecDeque<IpcBuffer>,
     reader: &mut R,
     block_offset: u64,
     is_little_endian: bool,
-    compression: Option<ipc::Message::BodyCompression>,
-) -> Result<Utf8Array<O>>
-where
-    Vec<u8>: TryInto<O::Bytes> + TryInto<<u8 as NativeType>::Bytes>,
-{
-    let field_node = field_nodes.pop_front().unwrap();
+    compression: Option<Compression>,
+) -> Result<Utf8Array<O>> {
+    let field_node = field_nodes.pop_front().ok_or_else(|| {
+        ArrowError::oos(format!(
+            "IPC: unable to fetch the field for {:?}. The file or stream is corrupted.",
+            data_type
+        ))
+    })?;
 
     let validity = read_validity(
         buffers,
@@ -45,7 +43,7 @@ where
         compression,
     )
     // Older versions of the IPC format sometimes do not report an offset
-    .or_else(|_| Result::Ok(Buffer::<O>::from(&[O::default()])))?;
+    .or_else(|_| Result::Ok(Buffer::<O>::from(vec![O::default()])))?;
 
     let last_offset = offsets.as_slice()[offsets.len() - 1].to_usize();
     let values = read_buffer(
@@ -62,10 +60,22 @@ where
     ))
 }
 
-pub fn skip_utf8(field_nodes: &mut VecDeque<Node>, buffers: &mut VecDeque<&ipc::Schema::Buffer>) {
-    let _ = field_nodes.pop_front().unwrap();
+pub fn skip_utf8(
+    field_nodes: &mut VecDeque<Node>,
+    buffers: &mut VecDeque<IpcBuffer>,
+) -> Result<()> {
+    let _ = field_nodes.pop_front().ok_or_else(|| {
+        ArrowError::oos("IPC: unable to fetch the field for utf8. The file or stream is corrupted.")
+    })?;
 
-    let _ = buffers.pop_front().unwrap();
-    let _ = buffers.pop_front().unwrap();
-    let _ = buffers.pop_front().unwrap();
+    let _ = buffers
+        .pop_front()
+        .ok_or_else(|| ArrowError::oos("IPC: missing validity buffer."))?;
+    let _ = buffers
+        .pop_front()
+        .ok_or_else(|| ArrowError::oos("IPC: missing offsets buffer."))?;
+    let _ = buffers
+        .pop_front()
+        .ok_or_else(|| ArrowError::oos("IPC: missing values buffer."))?;
+    Ok(())
 }
