@@ -1,5 +1,6 @@
 use crate::datatypes::DataType;
 use parquet2::statistics::BooleanStatistics as ParquetBooleanStatistics;
+use std::any::Any;
 
 use super::Statistics;
 
@@ -15,6 +16,14 @@ impl Statistics for BooleanStatistics {
     fn data_type(&self) -> &DataType {
         &DataType::Boolean
     }
+
+    fn as_any(&self) -> &dyn Any {
+        self
+    }
+
+    fn null_count(&self) -> Option<i64> {
+        self.null_count
+    }
 }
 
 impl From<&ParquetBooleanStatistics> for BooleanStatistics {
