@@ -1,9 +1,23 @@
+use std::sync::Arc;
+
 use super::*;
 use crate::types::days_ms;
 
-impl PartialEq for dyn Scalar {
-    fn eq(&self, other: &Self) -> bool {
-        equal(self, other)
+impl PartialEq for dyn Scalar + '_ {
+    fn eq(&self, that: &dyn Scalar) -> bool {
+        equal(self, that)
+    }
+}
+
+impl PartialEq<dyn Scalar> for Arc<dyn Scalar + '_> {
+    fn eq(&self, that: &dyn Scalar) -> bool {
+        equal(&**self, that)
+    }
+}
+
+impl PartialEq<dyn Scalar> for Box<dyn Scalar + '_> {
+    fn eq(&self, that: &dyn Scalar) -> bool {
+        equal(&**self, that)
     }
 }
 
@@ -111,7 +125,7 @@ fn equal(lhs: &dyn Scalar, rhs: &dyn Scalar) -> bool {
             let rhs = rhs.as_any().downcast_ref::<ListScalar<i64>>().unwrap();
             lhs == rhs
         }
-        DataType::Dictionary(key_type, _) => match_integer_type!(key_type, |$T| {
+        DataType::Dictionary(key_type, _, _) => match_integer_type!(key_type, |$T| {
             let lhs = lhs.as_any().downcast_ref::<DictionaryScalar<$T>>().unwrap();
             let rhs = rhs.as_any().downcast_ref::<DictionaryScalar<$T>>().unwrap();
             lhs == rhs
@@ -121,6 +135,6 @@ fn equal(lhs: &dyn Scalar, rhs: &dyn Scalar) -> bool {
             let rhs = rhs.as_any().downcast_ref::<StructScalar>().unwrap();
             lhs == rhs
         }
-        other => unimplemented!("{}", other),
+        other => unimplemented!("{:?}", other),
     }
 }
