@@ -3,13 +3,13 @@ use std::sync::Arc;
 use arrow2::{array::*, buffer::Buffer, datatypes::*, error::Result};
 
 #[test]
-fn display() -> Result<()> {
+fn debug() -> Result<()> {
     let fields = vec![
         Field::new("a", DataType::Int32, true),
         Field::new("b", DataType::Utf8, true),
     ];
     let data_type = DataType::Union(fields, None, UnionMode::Sparse);
-    let types = Buffer::from(&[0, 0, 1]);
+    let types = Buffer::from_slice([0, 0, 1]);
     let fields = vec![
         Arc::new(Int32Array::from(&[Some(1), None, Some(2)])) as Arc<dyn Array>,
         Arc::new(Utf8Array::<i32>::from(&[Some("a"), Some("b"), Some("c")])) as Arc<dyn Array>,
@@ -17,7 +17,7 @@ fn display() -> Result<()> {
 
     let array = UnionArray::from_data(data_type, types, fields, None);
 
-    assert_eq!(format!("{}", array), "UnionArray[1, , c]");
+    assert_eq!(format!("{:?}", array), "UnionArray[1, None, c]");
 
     Ok(())
 }
@@ -29,7 +29,7 @@ fn slice() -> Result<()> {
         Field::new("b", DataType::Utf8, true),
     ];
     let data_type = DataType::Union(fields, None, UnionMode::Sparse);
-    let types = Buffer::from(&[0, 0, 1]);
+    let types = Buffer::from_slice([0, 0, 1]);
     let fields = vec![
         Arc::new(Int32Array::from(&[Some(1), None, Some(2)])) as Arc<dyn Array>,
         Arc::new(Utf8Array::<i32>::from(&[Some("a"), Some("b"), Some("c")])) as Arc<dyn Array>,
@@ -39,7 +39,7 @@ fn slice() -> Result<()> {
 
     let result = array.slice(1, 2);
 
-    let types = Buffer::from(&[0, 1]);
+    let types = Buffer::from_slice([0, 1]);
     let expected = UnionArray::from_data(data_type, types, fields, None);
 
     assert_eq!(expected, result);
