@@ -2,45 +2,44 @@ use std::io::Cursor;
 use std::sync::Arc;
 
 use arrow2::array::*;
+use arrow2::chunk::Chunk;
 use arrow2::datatypes::*;
 use arrow2::error::Result;
 use arrow2::io::csv::write::*;
-use arrow2::record_batch::RecordBatch;
 
-fn data() -> RecordBatch {
+fn data() -> Chunk<Box<dyn Array>> {
     let c1 = Utf8Array::<i32>::from_slice(["a b", "c", "d"]);
     let c2 = Float64Array::from([Some(123.564532), None, Some(-556132.25)]);
     let c3 = UInt32Array::from_slice(&[3, 2, 1]);
     let c4 = BooleanArray::from(&[Some(true), Some(false), None]);
     let c5 = PrimitiveArray::<i64>::from([None, Some(1555584887378), Some(1555555555555)])
         .to(DataType::Timestamp(TimeUnit::Millisecond, None));
-    let c6 = PrimitiveArray::<i32>::from_slice(&[1234, 24680, 85563])
+    let c6 = PrimitiveArray::<i32>::from_vec(vec![1234, 24680, 85563])
         .to(DataType::Time32(TimeUnit::Second));
     let keys = UInt32Array::from_slice(&[2, 0, 1]);
     let c7 = DictionaryArray::from_data(keys, Arc::new(c1.clone()));
 
-    RecordBatch::try_from_iter(vec![
-        ("c1", Arc::new(c1) as Arc<dyn Array>),
-        ("c2", Arc::new(c2) as Arc<dyn Array>),
-        ("c3", Arc::new(c3) as Arc<dyn Array>),
-        ("c4", Arc::new(c4) as Arc<dyn Array>),
-        ("c5", Arc::new(c5) as Arc<dyn Array>),
-        ("c6", Arc::new(c6) as Arc<dyn Array>),
-        ("c7", Arc::new(c7) as Arc<dyn Array>),
+    Chunk::new(vec![
+        Box::new(c1) as Box<dyn Array>,
+        Box::new(c2),
+        Box::new(c3),
+        Box::new(c4),
+        Box::new(c5),
+        Box::new(c6),
+        Box::new(c7),
     ])
-    .unwrap()
 }
 
 #[test]
 fn write_csv() -> Result<()> {
-    let batch = data();
+    let columns = data();
 
     let write = Cursor::new(Vec::<u8>::new());
     let mut writer = WriterBuilder::new().from_writer(write);
 
-    write_header(&mut writer, batch.schema())?;
+    write_header(&mut writer, &["c1", "c2", "c3", "c4", "c5", "c6", "c7"])?;
     let options = SerializeOptions::default();
-    write_batch(&mut writer, &batch, &options)?;
+    write_chunk(&mut writer, &columns, &options)?;
 
     // check
     let buffer = writer.into_inner().unwrap().into_inner();
@@ -68,7 +67,7 @@ fn write_csv_custom_options() -> Result<()> {
         time64_format: Some("%r".to_string()),
         ..Default::default()
     };
-    write_batch(&mut writer, &batch, &options)?;
+    write_chunk(&mut writer, &batch, &options)?;
 
     // check
     let buffer = writer.into_inner().unwrap().into_inner();
@@ -83,7 +82,7 @@ d|-556132.25|1||2019-04-18 02:45:55.555|11:46:03 PM|c
     Ok(())
 }
 
-fn data_array(column: usize) -> (RecordBatch, Vec<&'static str>) {
+fn data_array(column: usize) -> (Chunk<Arc<dyn Array>>, Vec<&'static str>) {
     let (array, expected) = match column {
         0 => (
             Arc::new(Utf8Array::<i64>::from_slice(["a b", "c", "d"])) as Arc<dyn Array>,
@@ -114,32 +113,43 @@ fn data_array(column: usize) -> (RecordBatch, Vec<&'static str>) {
             vec!["3", "2", "1"],
         ),
         7 => (
-            Arc::new(UInt64Array::from_slice(&[3, 2, 1])) as Arc<dyn Array>,
+            Arc::new(UInt8Array::from_slice(&[3, 2, 1])) as Arc<dyn Array>,
             vec!["3", "2", "1"],
         ),
         8 => (
+            Arc::new(UInt16Array::from_slice(&[3, 2, 1])) as Arc<dyn Array>,
+            vec!["3", "2", "1"],
+        ),
+        9 => (
+            Arc::new(UInt32Array::from_slice(&[3, 2, 1])) as Arc<dyn Array>,
+            vec!["3", "2", "1"],
+        ),
+        10 => (
             Arc::new(UInt64Array::from_slice(&[3, 2, 1])) as Arc<dyn Array>,
             vec!["3", "2", "1"],
         ),
-        9 => {
-            let array = PrimitiveArray::<i32>::from_slice(&[1_234_001, 24_680_001, 85_563_001])
+        11 => {
+            let array = PrimitiveArray::<i32>::from_vec(vec![1_234_001, 24_680_001, 85_563_001])
                 .to(DataType::Time32(TimeUnit::Millisecond));
             (
                 Arc::new(array) as Arc<dyn Array>,
                 vec!["00:20:34.001", "06:51:20.001", "23:46:03.001"],
             )
         }
-        10 => {
-            let array =
-                PrimitiveArray::<i64>::from_slice(&[1_234_000_001, 24_680_000_001, 85_563_000_001])
-                    .to(DataType::Time64(TimeUnit::Microsecond));
+        12 => {
+            let array = PrimitiveArray::<i64>::from_vec(vec![
+                1_234_000_001,
+                24_680_000_001,
+                85_563_000_001,
+            ])
+            .to(DataType::Time64(TimeUnit::Microsecond));
             (
                 Arc::new(array) as Arc<dyn Array>,
                 vec!["00:20:34.000001", "06:51:20.000001", "23:46:03.000001"],
             )
         }
-        11 => {
-            let array = PrimitiveArray::<i64>::from_slice(&[
+        13 => {
+            let array = PrimitiveArray::<i64>::from_vec(vec![
                 1_234_000_000_001,
                 24_680_000_000_001,
                 85_563_000_000_001,
@@ -154,7 +164,7 @@ fn data_array(column: usize) -> (RecordBatch, Vec<&'static str>) {
                 ],
             )
         }
-        12 => {
+        14 => {
             let array = PrimitiveArray::<i64>::from_slice([
                 1_555_584_887_378_000_001,
                 1_555_555_555_555_000_001,
@@ -168,7 +178,7 @@ fn data_array(column: usize) -> (RecordBatch, Vec<&'static str>) {
                 ],
             )
         }
-        13 => {
+        15 => {
             let array = PrimitiveArray::<i64>::from_slice([
                 1_555_584_887_378_000_001,
                 1_555_555_555_555_000_001,
@@ -185,7 +195,14 @@ fn data_array(column: usize) -> (RecordBatch, Vec<&'static str>) {
                 ],
             )
         }
-        14 => {
+        16 => {
+            let keys = UInt32Array::from_slice(&[2, 1, 0]);
+            let values =
+                Arc::new(Utf8Array::<i64>::from_slice(["a b", "c", "d"])) as Arc<dyn Array>;
+            let array = DictionaryArray::from_data(keys, values);
+            (Arc::new(array) as Arc<dyn Array>, vec!["d", "c", "a b"])
+        }
+        17 => {
             let array = PrimitiveArray::<i64>::from_slice([
                 1_555_584_887_378_000_001,
                 1_555_555_555_555_000_001,
@@ -205,21 +222,19 @@ fn data_array(column: usize) -> (RecordBatch, Vec<&'static str>) {
         _ => todo!(),
     };
 
-    (
-        RecordBatch::try_from_iter(vec![("c1", array)]).unwrap(),
-        expected,
-    )
+    (Chunk::new(vec![array]), expected)
 }
 
-fn write_single(column: usize) -> Result<()> {
-    let (batch, data) = data_array(column);
-
+fn test_array(
+    columns: Chunk<Arc<dyn Array>>,
+    data: Vec<&'static str>,
+    options: SerializeOptions,
+) -> Result<()> {
     let write = Cursor::new(Vec::<u8>::new());
     let mut writer = WriterBuilder::new().delimiter(b'|').from_writer(write);
 
-    write_header(&mut writer, batch.schema())?;
-    let options = SerializeOptions::default();
-    write_batch(&mut writer, &batch, &options)?;
+    write_header(&mut writer, &["c1"])?;
+    write_chunk(&mut writer, &columns, &options)?;
 
     // check
     let buffer = writer.into_inner().unwrap().into_inner();
@@ -227,13 +242,19 @@ fn write_single(column: usize) -> Result<()> {
     let mut expected = "c1\n".to_owned();
     expected.push_str(&data.join("\n"));
     expected.push('\n');
-    assert_eq!(expected, String::from_utf8(buffer).unwrap(),);
+    assert_eq!(expected, String::from_utf8(buffer).unwrap());
     Ok(())
 }
 
+fn write_single(column: usize) -> Result<()> {
+    let (columns, data) = data_array(column);
+
+    test_array(columns, data, SerializeOptions::default())
+}
+
 #[test]
 fn write_each() -> Result<()> {
-    for i in 0..=13 {
+    for i in 0..=16 {
         write_single(i)?;
     }
     Ok(())
@@ -242,5 +263,54 @@ fn write_each() -> Result<()> {
 #[test]
 #[cfg(feature = "chrono-tz")]
 fn write_tz_timezone() -> Result<()> {
-    write_single(14)
+    write_single(17)
+}
+
+#[test]
+fn write_tz_timezone_formatted_offset() -> Result<()> {
+    let array =
+        PrimitiveArray::<i64>::from_slice([1_555_584_887_378_000_001, 1_555_555_555_555_000_001])
+            .to(DataType::Timestamp(
+                TimeUnit::Nanosecond,
+                Some("+01:00".to_string()),
+            ));
+
+    let columns = Chunk::new(vec![Arc::new(array) as Arc<dyn Array>]);
+    let expected = vec![
+        "2019-04-18T11:54:47.378000001+01:00",
+        "2019-04-18T03:45:55.555000001+01:00",
+    ];
+    test_array(
+        columns,
+        expected,
+        SerializeOptions {
+            timestamp_format: Some("%Y-%m-%dT%H:%M:%S%.f%:z".to_string()),
+            ..Default::default()
+        },
+    )
+}
+
+#[test]
+#[cfg(feature = "chrono-tz")]
+fn write_tz_timezone_formatted_tz() -> Result<()> {
+    let array =
+        PrimitiveArray::<i64>::from_slice([1_555_584_887_378_000_001, 1_555_555_555_555_000_001])
+            .to(DataType::Timestamp(
+                TimeUnit::Nanosecond,
+                Some("Europe/Lisbon".to_string()),
+            ));
+
+    let columns = Chunk::new(vec![Arc::new(array) as Arc<dyn Array>]);
+    let expected = vec![
+        "2019-04-18T11:54:47.378000001+01:00",
+        "2019-04-18T03:45:55.555000001+01:00",
+    ];
+    test_array(
+        columns,
+        expected,
+        SerializeOptions {
+            timestamp_format: Some("%Y-%m-%dT%H:%M:%S%.f%:z".to_string()),
+            ..Default::default()
+        },
+    )
 }
